@@ -3,20 +3,23 @@
 """Initialization of Tinyscript package.
 
 """
+from warnings import filterwarnings
+filterwarnings("ignore", "Setuptools is replacing distutils.")
+
 
 from .__info__ import __author__, __copyright__, __license__, __version__
 
+from .deprecation import *
 from .features import *
 from .helpers import *
 from .parser import *
 from .preimports import *
-from .warnings import *
 
+from .deprecation import __features__ as _deprecation
 from .features import __features__ as _features
 from .helpers import __features__ as _helpers
 from .parser import __features__ as _parser
 from .preimports import __features__ as _preimports
-from .warnings import __features__ as _warnings
 
 
 ts.__author__    = __author__
@@ -25,5 +28,5 @@ ts.__license__   = __license__
 ts.__version__   = __version__
 
 
-__all__ = _features + _helpers + _parser + _preimports + _warnings
+__all__ = _deprecation + _features + _helpers + _parser + _preimports
 
