@@ -11,7 +11,7 @@ from argparse import _ActionsContainer, _ArgumentGroup, _MutuallyExclusiveGroup,
                      ArgumentDefaultsHelpFormatter, Action, ArgumentError, RawTextHelpFormatter, SUPPRESS, \
                      _UNRECOGNIZED_ARGS_ATTR, Namespace as BaseNamespace, ArgumentParser as BaseArgumentParser
 from os import environ
-from os.path import abspath, basename, dirname, splitext
+from os.path import abspath, basename, dirname, sep, splitext
 from stat import S_IXUSR
 try:
     from configparser import ConfigParser, NoOptionError, NoSectionError
@@ -23,8 +23,10 @@ from .helpers.constants import PYTHON3
 from .helpers.inputs import user_input
 from .helpers.licenses import *
 from .helpers.data.types import is_executable, is_long_opt, is_pos_int, is_short_opt
+from .helpers.termsize import get_terminal_size
 from .helpers.text import *
 from .helpers.text import configure_docformat, txt_terminal_render
+from .preimports.shutilp import which
 
 
 __all__ = ["ArgumentParser", "DUNDERS", "SCRIPTNAME_FORMAT", "SUPPRESS"]
@@ -40,13 +42,14 @@ DUNDERS = BASE_DUNDERS + [
 
 DEFAULT_MAX_LEN     = 20
 DEFAULT_LST_MAX_LEN = 10
-SCRIPTNAME_FORMAT   = "slugified"
+SCRIPTNAME_FORMAT   = "camelcase"
 SCRIPTNAME_FORMATS  = {
-    'acronym':   lambda s: "".join(x.strip()[0].upper() for x in re.split(r"[ -_]", s)) \
-                           if len(re.split(r"[ -_]", s)) > 1 else s.upper(),
+    'acronym':   lambda s: "".join(x.strip()[0].upper() for x in re.split(r"[ -_]", s.lower())) \
+                           if len(re.split(r"[ -_]", s.lower())) > 1 else s.upper(),
     'as_is':     lambda s: s,
+    'camelcase': lambda s: "".join(x.strip().capitalize() for x in re.split(r"[ -_]", s)),
     'none':      lambda s: s,
-    'slugified': lambda s: "".join(x.strip().capitalize() for x in re.split(r"[ -_]", s)),
+    'slugified': lambda s: "-".join(x.strip().lower() for x in re.split(r"[ -_]", s)),
 }
 
 
@@ -56,7 +59,7 @@ class _ConfigAction(Action):
     def __init__(self, option_strings, dest=None, default=None, help=None):
         super(_ConfigAction, self).__init__(option_strings=option_strings, dest=SUPPRESS, default=default, nargs=1,
                                             help=gt(help), metavar="INI")
-
+    
     def __call__(self, parser, namespace, values, option_string=None):
         conf = values[0]
         setattr(namespace, "read_config", conf)
@@ -71,7 +74,7 @@ class _DemoAction(Action):
     def __init__(self, option_strings, dest=SUPPRESS, help=None):
         super(_DemoAction, self).__init__(option_strings=option_strings, dest=SUPPRESS, default=SUPPRESS, nargs=0,
                                           help=gt(help))
-
+    
     def __call__(self, parser, namespace, values, option_string=None):
         parser.demo_args()
 
@@ -79,11 +82,15 @@ class _DemoAction(Action):
 class _ExtendAction(Action):
     """ Custom action for extending a list of values. """
     def __call__(self, parser, namespace, values, option_string=None):
-        _ = getattr(namespace, self.dest) or []
+        l = getattr(namespace, self.dest) or []
+        if not isinstance(l, list):
+            if not getattr(namespace, "default_silent", False):
+                logger.warning(gt("extend is used with %s while its value is not a list").format(self.dest))
+            l = []
         if not isinstance(values, list):
             values = [values]
-        _.extend(values)
-        setattr(namespace, self.dest, _)
+        l.extend(values)
+        setattr(namespace, self.dest, l)
 
 
 class _NewSubParsersAction(_SubParsersAction):
@@ -110,7 +117,7 @@ class _NewSubParsersAction(_SubParsersAction):
         # create the parser, but with another formatter and separating the help into an argument group
         parser = self._parser_class(ArgumentParser.globals_dict, add_help=False, **kwargs)
         parser.name = name
-        i = parser.add_argument_group(txt2title(gt("extra arguments")))
+        i = parser.add_argument_group(gt("extra arguments"))
         i.add_argument("-h", action="usage", prefix="show", help=gt("show usage message and exit"))
         i.add_argument("--help", action="help", prefix="show", help=gt("show this help message and exit"))
         # add it to the map
@@ -126,7 +133,7 @@ class _UsageAction(Action):
     def __init__(self, option_strings, dest=SUPPRESS, help=None):
         super(_UsageAction, self).__init__(option_strings=option_strings, dest=SUPPRESS, default=SUPPRESS, nargs=0,
                                            help=gt(help))
-
+    
     def __call__(self, parser, namespace, values, option_string=None):
         parser.print_usage()
         parser.exit()
@@ -203,12 +210,12 @@ class _NewActionsContainer(_ActionsContainer):
                     long_opt = "{}-{}".format(long_opt, suffix)
                     args.append(long_opt)
                     return self.add_argument(*args, **kwargs)
-
+    
     def add_argument_group(self, *args, **kwargs):
         group = _NewArgumentGroup(self, *args, **kwargs)
         self._action_groups.append(group)
         return group
-
+    
     def add_mutually_exclusive_group(self, **kwargs):
         group = _NewMutuallyExclusiveGroup(self, **kwargs)
         self._mutually_exclusive_groups.append(group)
@@ -251,19 +258,26 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
         self._docfmt = gd.get('__docformat__')
         self._reparse_args = {'pos': [], 'opt': [], 'sub': []}
         self.examples = gd.get('__examples__', [])
-        script = gd.get('__script__', gd.get('__file__', sys.argv[0]))
+        script = basename(gd.get('__script__', gd.get('__file__', sys.argv[0])))
         if script and kwargs.get('prog') is None:
-            path = abspath(script)
+            path = abspath(which(script) or script)
             root = dirname(path)
             script = basename(script)
-            kwargs['prog'] = script if gd.get('__script__') else \
-                             "python{} ".format(["", "3"][PYTHON3]) + script if not is_executable(path) else \
-                             "./" + script if root not in environ['PATH'].split(":") else script
+            if gd.get('__script__'):
+                kwargs['prog'] = gd['__script__']
+            elif not is_executable(path):
+                kwargs['prog'] = "python{} ".format(["", "3"][PYTHON3]) + script
+            elif root not in [x.rstrip(sep) for x in environ['PATH'].split(":")]:
+                kwargs['prog'] = "./" + script
+            else:
+                kwargs['prog'] = splitext(script)[0]
             ArgumentParser.prog = kwargs['prog']
-            script, _ = splitext(script)
         kwargs['add_help'] = False
         kwargs['conflict_handler'] = "error"
-        kwargs['formatter_class'] = HelpFormatter
+        # when __docformat__ is set, fixing max_help_position to terminal's width forces argparse to format arguments
+        #  with their help on the same line ; in other words, formatting is left to the renderer
+        kwargs['formatter_class'] = HelpFormatter if self._docfmt is None else \
+                                    lambda prog: HelpFormatter(prog, max_help_position=get_terminal_size()[0])
         # format the epilog message
         if self.examples and script:
             _ = ["{} {}".format(ArgumentParser.prog, e) for e in self.examples]
@@ -275,10 +289,9 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
         # adapt the script name ; if SCRIPTNAME is provided, it supersedes SCRIPTNAME_FORMAT, otherwise compute the name
         #  according to the format specified in SCRIPTNAME_FORMAT
         sname = script
-        sname_fmt  = gd.get('SCRIPTNAME_FORMAT', SCRIPTNAME_FORMAT)
-        sname_func = SCRIPTNAME_FORMATS.get(sname_fmt)
+        sname_func = SCRIPTNAME_FORMATS.get(gd.get('SCRIPTNAME_FORMAT', SCRIPTNAME_FORMAT))
         if sname_func:
-            sname = sname_func(script)
+            sname = sname_func(splitext(script)[0])
         else:
             l = "\n- ".join(sorted(SCRIPTNAME_FORMATS.keys()))
             raise ValueError("Bad script name format ; please use one of the followings:\n{}".format(l))
@@ -485,6 +498,8 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
             formatter.add_arguments(action_group._group_actions)
             formatter.end_section()
             actions = formatter.format_help()
+            if actions == "":
+                continue
             a = ""
             for i, line in enumerate(actions.splitlines()):
                 s, dedent = line.lstrip(), 0
@@ -492,21 +507,21 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
                     # action_group.title has ":" added after txt2title(...) causing an extra ":" to be appended
                     if self._docfmt == "html":
                         # for markup languages with tags, ":" appears behind the title tag
-                        a += line.rstrip(":") + "\n"
+                        title = line.rstrip(":") + "\n"
                     else:
                         # for other markup languages without tags, ":" appears at the end in duplicate
-                        a += line.rstrip(":") + ":\n"
+                        title = line.rstrip(":") + ":\n"
                     continue
                 if i == 1 and self._docfmt == "rst":
                     # action_group.title has ":" added after txt2title(...) causing an extra ":" to be appended behind
                     #  the underline, making rendering fail
-                    a = a.rstrip("\n") + "\n" + line.rstrip(":") + "\n"
+                    title = title.rstrip("\n") + "\n" + line.rstrip(":") + "\n"
                     continue
                 if self._docfmt and s[0] in "-*":
                     dedent = len(line) - len(s)
-                a += (line + "\n\n") if line.rstrip()[-1] == ":" else (txt2paragraph(line[dedent:]) + \
-                                                                       ["\n\n", "\n"][self._docfmt is None])
-            text += a + "\n"
+                _nl = ["\n", ""][self._docfmt is None]
+                a += _nl + txt2paragraph(line.lstrip() if self._docfmt else line) + "\n"
+            text += title + a + "\n"
         # epilog
         formatter = self._get_formatter()
         formatter.add_text(self.epilog)
@@ -519,8 +534,9 @@ class ArgumentParser(_NewActionsContainer, BaseArgumentParser):
         formatter.add_usage(self.usage, self._actions, self._mutually_exclusive_groups)
         text = formatter.format_help()
         if self._docfmt:
-            title, usage = text.split(": ", 1)
-            text = txt2title(title + ":") + "\n\n" + txt2paragraph(usage.rstrip("\n")) + "\n"
+            title, usage = text.rstrip("\n").split(": ", 1)
+            usage = "\n".join(l[2:] if i > 0 else l for i, l in enumerate(usage.split("\n")))
+            text = txt2title(title + ":") + "\n\n" + txt2paragraph(usage) + "\n"
         return text
     
     def input_args(self):
@@ -601,7 +617,7 @@ class HelpFormatter(ArgumentDefaultsHelpFormatter, RawTextHelpFormatter):
                     s = ','.join(p[:2] + ["..."] + p[-2:])
             params['default'] = s
         return self._get_help_string(action) % params
-
+    
     def _get_help_string(self, action):
         help = super(HelpFormatter, self)._get_help_string(action)
         if '%(note)' not in help and hasattr(action, "note") and action.note is not None:
