@@ -14,6 +14,7 @@ from .data import *
 from .decorators import *
 from .dictionaries import *
 from .docstring import *
+from .expressions import *
 from .fexec import *
 from .inputs import *
 from .layout import *
@@ -34,6 +35,7 @@ from .data import __features__ as _data
 from .decorators import __features__ as _dec
 from .dictionaries import __features__ as _dict
 from .docstring import __features__ as _docs
+from .expressions import __features__ as _expr
 from .fexec import __features__ as _fexec
 from .inputs import __features__ as _inputs
 from .layout import __features__ as _layout
@@ -46,8 +48,8 @@ from .text import __features__ as _text
 from .timeout import __features__ as _to
 
 
-__helpers__ = _attack + _common + _data + _dec + _dict + _docs + _fexec + _inputs + _layout + _lic + _notify + _path + \
-              _pswd + _tsize + _text + _to
+__helpers__ = _attack + _common + _data + _dec + _dict + _docs + _expr + _fexec + _inputs + _layout + _lic + _notify + \
+              _path + _pswd + _tsize + _text + _to
 
 ts = ModuleType("ts", """
     Tinyscript helpers
@@ -80,7 +82,7 @@ ts = ModuleType("ts", """
       
     - Path-related helpers
       
-      Some useful path functions, enhanced from pathlib.Path's one. It adds
+      Some useful path functions, enhanced from pathlib2.Path's one. It adds
       various methods to Path and provides helpers for mirroring a path or
       creating a temporary one.
       
