@@ -3,6 +3,7 @@
 
 """
 import sys
+from getpass import getuser
 from locale import getlocale
 from platform import system
 
@@ -12,15 +13,22 @@ DARWIN  = system() == "Darwin"
 LINUX   = system() == "Linux"
 WINDOWS = system() == "Windows"
 
+__all__ += ["ADMIN", "USER"]
+USER = getuser()
+ADMIN = USER == ["root", "Administrator"][WINDOWS]
+
 __all__ += ["ENCODING", "LANGUAGE"]
 LANGUAGE, ENCODING = getlocale()
 
-__all__ += ["IPYTHON", "JUPYTER", "JYTHON", "PYPY", "PYTHON3"]
+__all__ += ["IPYTHON", "JUPYTER", "JYTHON", "PYPY", "PYTHON2", "PYTHON3", "TTY"]
 JYTHON  = sys.platform.startswith("java")
 PYPY    = hasattr(sys, "pypy_version_info")
+PYTHON2 = sys.version_info < (3,)
 PYTHON3 = sys.version_info > (3,)
-IPYTHON = JUPYTER = True
+TTY     = sys.stdout.isatty()
 try:
     __IPYTHON__
+    IPYTHON = JUPYTER = True
 except NameError:
     IPYTHON = JUPYTER = False
+
