@@ -9,6 +9,7 @@ from datetime import datetime
 __all__ = __features__ = ["copyright", "license", "list_licenses"]
 
 
+# source: https://docs.github.com/en/github/creating-cloning-and-archiving-repositories/licensing-a-repository
 LICENSES = {
     'afl-3.0': "Academic Free License v3.0",
     'agpl-3.0': "GNU Affero General Public License v3.0",
@@ -46,11 +47,13 @@ LICENSES = {
 }
 
 
-def copyright(text, start=None):
+def copyright(text, start=None, end=None):
     """ Make the copyright field value from the given text. """
     year = datetime.now().year
-    if isinstance(start, int) and start < year:
-        year = "{}-{}".format(start, year)
+    if end is None:
+        end = year
+    if isinstance(start, int) and isinstance(end, int) and start < end:
+        year = "{}-{}".format(start, end)
     return "© {} {}".format(year, text) if not text.startswith("©") else text
 
 
