@@ -1,56 +1,61 @@
 # -*- coding: UTF-8 -*-
-"""Path-oriented structures derived from pathlib.Path.
+"""Path-oriented structures derived from pathlib2.Path.
 
 """
 import ctypes
+import errno
 import imp
 import importlib
 import os
 import re
 from mimetypes import guess_type
-from pathlib import Path as BasePath
+from pathlib2 import Path as BasePath
 from pyminizip import compress_multiple, uncompress
 from random import choice
 from re import search
-from shutil import rmtree
+from shutil import copy, copy2, copytree, rmtree
 from six import string_types
 from tempfile import gettempdir, NamedTemporaryFile as TempFile
 
 from .constants import *
 from .compat import u
-from .password import getpass
-
-if PYTHON2:
-    from pygments.lexers import Python2Lexer as PythonLexer
-else:
-    from pygments.lexers import PythonLexer
+from .data.types import is_dict, is_list, is_str
+from .inputs import user_input
+from .password import getpass, getrepass
 
 
-__all__ = __features__ = ["Path", "ConfigPath", "MirrorPath", "ProjectPath", "PythonPath", "TempPath"]
+__all__ = __features__ = ["Path", "ConfigPath", "CredentialsPath", "MirrorPath", "ProjectPath", "PythonPath",
+                          "TempPath"]
 
 
-MARKER = "#TODO:"
-
-# NB: PythonLexer.analyse_text only relies on shebang !
-_lexer = PythonLexer()
+DOUBLE_EXT = re.compile(r"^(.*?)(\.tar\.(?:br|bz2?|gz|lpaq|lzo?|xz|zst|Z))$")
+MARKER     = "#TODO:"
 
 
 class Path(BasePath):
-    """ Extension of the base class Path from pathlib.
+    """ Extension of the base class Path from pathlib2.
     
     :param expand: expand user's path
     :param create: create the directory if it doesn't exist
+    :param touch:  create the file if it doesn't exist (mutually exclusive with 'create')
     """
+
     _flavour = BasePath()._flavour  # fix to AttributeError
     
     def __new__(cls, *parts, **kwargs):
         expand = kwargs.pop("expand", False)
         create = kwargs.pop("create", False)
+        touch  = kwargs.pop("touch", False)
         p = super(Path, cls).__new__(cls, *parts, **kwargs)
         if expand:
-            p = p.expanduser().absolute()
-        if create and not p.exists():
-            p.mkdir(parents=True)  # exist_ok does not work in Python 2
+            p = super(Path, cls).__new__(cls, str(p.expanduser().absolute()), **kwargs)
+        if create and touch:
+            raise ValueError("Conflicting options ; 'create' creates a folder hwile 'touch' creates a file")
+        elif (create or touch) and not p.exists():
+            if create:
+                p.mkdir(parents=True)  # exist_ok does not work in Python 2
+            elif touch:
+                p.touch()
         return p
     
     @property
@@ -74,16 +79,26 @@ class Path(BasePath):
         """ Get the directory name. """
         return self if self.is_dir() else self.parent
     
+    @property
+    def extension(self):
+        """ Get the extension based on the stem and suffix. """
+        return self.suffix
+    
     @property
     def filename(self):
         """ Get the file name, without the complete path. """
-        return self.stem + self.suffix
+        return self.basename
     
     @property
     def mime_type(self):
         """ Get the MIME type of the current Path object. """
         return guess_type(str(self))[0]
     
+    @property
+    def permissions(self):
+        """ Get the permissions of the current Path object. """
+        return os.stat(str(self)).st_mode
+    
     @property
     def size(self):
         """ Get path's size. """
@@ -97,6 +112,22 @@ class Path(BasePath):
                     s += os.stat(str(Path(root).joinpath(f))).st_size
             return s
     
+    @property
+    def stem(self):
+        """ Stem also handling some common double extensions. """
+        try:
+            return DOUBLE_EXT.search(self.basename).group(1)
+        except AttributeError:
+            return super(Path, self).stem
+    
+    @property
+    def suffix(self):
+        """ Suffix also handling some common double extensions. """
+        try:
+            return DOUBLE_EXT.search(self.basename).group(2)
+        except AttributeError:
+            return super(Path, self).suffix
+    
     @property
     def text(self):
         """ Get file's content as a string. """
@@ -111,7 +142,7 @@ class Path(BasePath):
             return f.write(u(data))
     
     def append_bytes(self, data):
-        """ Allows to append bytes to the file, as only write_bytes is available in pathlib, overwritting the former
+        """ Allows to append bytes to the file, as only write_bytes is available in pathlib2, overwritting the former
              bytes at each write. """
         with self.open(mode='ab') as f:
             return f.write(memoryview(data))
@@ -126,7 +157,7 @@ class Path(BasePath):
             self.append_line(line)
     
     def append_text(self, text, encoding=None, errors=None):
-        """ Allows to append text to the file, as only write_text is available in pathlib, overwritting the former text
+        """ Allows to append text to the file, as only write_text is available in pathlib2, overwritting the former text
              at each write. """
         return self.__add_text(text, 'a', encoding, errors)
     
@@ -142,6 +173,17 @@ class Path(BasePath):
             if len(l) > 0:
                 return self.joinpath(choice(l))
     
+    def copy(self, new_path, **kwargs):
+        """ Copy this folder or file to the given destination. """
+        try:
+            copytree(str(self), str(new_path), **kwargs)
+        except OSError as e:  # does not use NotADirectoryError as it is only available from Python3
+            if e.errno == errno.ENOTDIR:
+                (copy2 if kwargs.pop('metadata', True) else copy)(str(self), str(new_path), **kwargs)
+            else:
+                return self
+        return self.__class__(new_path)
+    
     def expanduser(self):
         """ Fixed expanduser() method, working for both Python 2 and 3. """
         return Path(os.path.expanduser(str(self)))
@@ -165,11 +207,12 @@ class Path(BasePath):
     
     def generate(self, prefix="", suffix="", length=8, alphabet="0123456789abcdef"):
         """ Generate a random folder name. """
-        if not self.is_dir():
+        # simply return self if it exists and it is not a directory
+        if self.exists() and not self.is_dir():
             return self
+        # ensure this is a newly generated path
         while True:
-            _ = "".join(choice(alphabet) for i in range(length))
-            new = self.joinpath(str(prefix) + _ + str(suffix))
+            new = self.joinpath(str(prefix) + "".join(choice(alphabet) for i in range(length)) + str(suffix))
             if not new.exists():
                 return new
     rand_folder_name = generate
@@ -212,7 +255,10 @@ class Path(BasePath):
     
     def listdir(self, filter_func=lambda p: True, sort=True):
         """ List the current path using the given filter. """
-        l = os.listdir(str(self))
+        try:
+            l = os.listdir(str(self))
+        except OSError:
+            return
         if sort:
             l = sorted(l)
         for item in l:
@@ -240,12 +286,13 @@ class Path(BasePath):
         with self.open('w') as f:
             pass
     
-    def remove(self):
+    def remove(self, error=True):
         """ Extension for removing a directory or a file. """
-        if self.is_dir():
-            rmtree(str(self))
-        else:
-            os.remove(str(self))
+        try:
+            rmtree(str(self)) if self.is_dir() else os.remove(str(self))
+        except OSError:
+            if error:
+                raise
     
     def walk(self, breadthfirst=True, filter_func=lambda p: True, sort=True, base_cls=True, relative=False):
         """ Walk the current path for directories and files using os.listdir(), breadth-first or depth-first, sorted or
@@ -257,9 +304,11 @@ class Path(BasePath):
                 if filter_func(item):
                     yield out(rel(item))
         for item in self.listdir(lambda p: p.is_dir(), sort):
+            if self.is_symlink() and self.resolve() == self.parent:
+                continue  # e.g.: /usr/bin/X11 -> /usr/bin
             if breadthfirst and filter_func(item):
                 yield out(rel(item))
-            for subitem in item.walk(breadthfirst, filter_func):
+            for subitem in item.walk(breadthfirst, filter_func, sort, base_cls):
                 yield out(rel(subitem))
             if not breadthfirst and filter_func(item):
                 yield out(rel(item))
@@ -268,10 +317,15 @@ class Path(BasePath):
                 if filter_func(item):
                     yield out(rel(item))
     
+    def write_bytes(self, data):
+        """ Fix to non-existing method in Python 2. """
+        with self.open(mode='wb') as f:
+            return f.write(memoryview(data))
+    
     def write_text(self, data, encoding=None, errors=None):
         """ Fix to non-existing method in Python 2. """
         return self.__add_text(data, 'w', encoding, errors)
-
+    
 
 class ConfigPath(Path):
     """ Extension of the class Path for handling a temporary path.
@@ -291,6 +345,86 @@ class ConfigPath(Path):
         return self
 
 
+class CredentialsPath(Path):
+    """ Extension of the class Path for handling a path to a file containing credentials.
+    
+    Note: Beware that this does not handle any security (no obfuscation, hashing or encryption) ; so, any secret entered
+           in an instance will remain in memory until this is freed.
+    
+    :param id:     identifier
+    :param secret: secret
+    """
+    def __new__(cls, *parts, **kwargs):
+        p = Path(*parts)
+        path, fn = (str(p), "creds.txt") if p.suffix == "" else (p.dirname, p.filename)
+        kw = {'create': True, 'expand': True}
+        path = Path(*parts, **kw)
+        kwargs['exist_ok'] = True
+        kwargs['create'] = False
+        self = super(CredentialsPath, cls).__new__(cls, str(path), fn, **kwargs)
+        self.id = kwargs.get("id") or ""
+        self.secret = kwargs.get("secret") or ""
+        d = kwargs.get("delimiter") or ":"
+        self.save(d) if self.id != "" and self.secret != "" else self.load(d)
+        return self
+    
+    def ask(self, id="Username:", secret="Password:"):
+        """ This method allows to ask for credentials.
+        
+        :param id:     prompt message or (prompt message, validation pattern)
+        :param secret: prompt message or (prompt message, validation pattern) or (prompt message, policy dictionary)
+        """
+        if is_str(id):
+            id_prompt, id_pattern = id, None
+        elif is_list(id) and len(id) == 2:
+            id_prompt, id_pattern = id
+        else:
+            raise ValueError("Bad identifier ; should be either the prompt message or a 2-tuple with the prompt message"
+                             " and the validation pattern")
+        id_alias = id_prompt.rstrip(": ").lower().replace(" ", "_")
+        if is_str(secret):
+            sec_prompt, sec_pattern = secret, None
+        elif is_list(secret) and len(secret) == 2:
+            sec_prompt, sec_pattern = secret
+        else:
+            raise ValueError("Bad secret ; should be either the prompt message, a 2-tuple with the prompt message"
+                             " and the validation pattern or a 2-tuple with the prompt message and a policy")
+        sec_alias = sec_prompt.rstrip(": ").lower().replace(" ", "_")
+        self.id = ""
+        while self.id == "":
+            self.id = user_input(id_prompt, required=True)
+            if id_pattern and not re.search(id_pattern, self.id):
+                raise ValueError("Bad %s" % id_alias)
+        self.secret = ""
+        try:
+            self.secret = (getpass if is_dict(sec_pattern) else getrepass)(sec_prompt, None, sec_pattern).strip()
+        except ValueError as e:
+            raise e("Non-compliant %s:\n- %s" % (sec_alias, "\n- ".join(e.errors))) if hasattr(e, "errors") else e
+    
+    def load(self, delimiter=":"):
+        """ This loads credentials from the path taking a delimiter (default: ":") into account. """
+        if not self.exists():
+            return
+        c = self.read_text().strip()
+        if c == "":
+            return
+        try:
+            self.id, self.secret = c.split(delimiter)
+        except ValueError:
+            raise ValueError("Bad delimiter '%s' ; it should not be a character present in the identifier or secret" % \
+                             delimiter)
+    
+    def save(self, delimiter=":"):
+        """ This saves the defined credentials to the path taking a delimiter (default: ":") into account. """
+        if self.id == "" and self.secret == "":
+            return
+        self.touch(0o600)
+        if delimiter in self.id or delimiter in self.secret:
+            raise ValueError("Bad delimiter '%s' ; it should not be a character present in the identifier or secret" % \
+                             delimiter)
+        self.write_text("%s%s%s" % (self.id, delimiter, self.secret))
+
+
 class MirrorPath(Path):
     """ Extension of the class Path for handling a folder that can mirror another one using symbolic links.
     
@@ -336,7 +470,7 @@ class ProjectPath(Path):
         return self
     
     def archive(self, path=None, password=None, ask=False, remove=True, **kwargs):
-        """ This function compresses the content of the given source path into the given archive as the destination
+        """ This method compresses the content of the given source path into the given archive as the destination
              path, eventually given a password.
 
         :param path:     path to the project folder to be archived
@@ -373,7 +507,7 @@ class ProjectPath(Path):
                 sp.write_text(str(v))
     
     def load(self, path=None, password=None, ask=False, remove=True, **kwargs):
-        """ This function decompresses the given archive, eventually given a password.
+        """ This method decompresses the given archive, eventually given a password.
 
         :param path:     path to the archive to be extracted
         :param password: password string to be passed
@@ -398,22 +532,25 @@ class ProjectPath(Path):
         matches = {}
         for p in self.walk(filter_func=lambda p: p.is_file()):
             current_cls, line_number = None, None
-            with p.open() as f:
-                for i, l in enumerate(f):
-                    if "class " in l:
-                        current_cls = l.split("class ")[1].split("(")[0].strip()
-                    if "def " in l:
-                        line_number = str(i + 1)
-                    match = re.search(pattern, l)
-                    if match:
-                        m = str(Path(p))
-                        if current_cls is not None:
-                            m += ":" + current_cls
-                        m += ":" + (line_number or str(i + 1))
-                        try:
-                            matches[m] = match.group(1)
-                        except IndexError:
-                            matches[m] = l
+            try:
+                with p.open() as f:
+                    for i, l in enumerate(f):
+                        if "class " in l:
+                            current_cls = l.split("class ")[1].split("(")[0].strip()
+                        if "def " in l:
+                            line_number = str(i + 1)
+                        match = re.search(pattern, l)
+                        if match:
+                            m = str(Path(p))
+                            if current_cls is not None:
+                                m += ":" + current_cls
+                            m += ":" + (line_number or str(i + 1))
+                            try:
+                                matches[m] = match.group(1)
+                            except IndexError:
+                                matches[m] = l
+            except UnicodeDecodeError:
+                pass
         return matches
     
     @property
@@ -429,21 +566,37 @@ class ProjectPath(Path):
 
 class PythonPath(Path):
     """ Path extension for handling the dynamic import of Python modules. """
-    def __init__(self, path):
+    def __init__(self, path, remove_cache=False):
         super(PythonPath, self).__init__()
-        try:
-            self.is_pymodule = self.is_file() and _lexer.analyse_text(self.open().read()) == 1.0
-        except UnicodeDecodeError:
-            self.is_pymodule = False
+        if remove_cache:
+            f = (lambda x: x.is_file() and x.extension == ".pyc") if PYTHON2 else \
+                (lambda x: x.is_dir() and x.basename == "__pycache__")
+            for p in self.walk(filter_func=f):
+                p.remove(False)
         if self.is_dir():
             self.modules = []
-            for p in self.walk(base_cls=False):
-                p = PythonPath(p)
-                if p.is_pymodule:
-                    self.modules.append(p.module)
+            _cached = []
+            for e in [".pyc", ".py"]:
+                for p in self.walk(filter_func=lambda x: x.extension == e, base_cls=False):
+                    if not p.is_file() or str(p.absolute()) in _cached:
+                        continue
+                    if e == ".pyc" and (PYTHON3 and p.absolute().dirname.parts[-1] == "__pycache__" or PYTHON2):
+                        parts = p.filename.split(".")
+                        if PYTHON3 and len(parts) == 3 and re.match(r".?python\-?[23]\d", parts[-2]) or PYTHON2:
+                            d = p.absolute().dirname
+                            d = d if PYTHON2 else d.parent
+                            _cached.append(str(d.joinpath("%s.py" % parts[0])))
+                    p = PythonPath(p)
+                    if p.loaded:
+                        self.modules.append(p.module)
         else:
-            if self.is_pymodule:
-                self.module = imp.load_source(self.stem, str(self))
+            self.loaded = False
+            if self.extension in [".py", ".pyc"]:
+                try:
+                    self.module = [imp.load_compiled, imp.load_source][self.extension == ".py"](self.stem, str(self))
+                    self.loaded = True
+                except (ImportError, NameError, SyntaxError, ValueError):
+                    pass
     
     @property
     def classes(self):
@@ -503,20 +656,19 @@ class TempPath(Path):
             kw["length"]   = kwargs.pop("length", 0)
             kw["alphabet"] = kwargs.pop("alphabet", "0123456789abcdef")
             if kw["length"] > 0:
-                while True:
-                    # ensure this is a newly generated path
-                    tmp = p.generate(**kw)
-                    if not tmp.exists():
-                        break
-                return super(TempPath, cls).__new__(cls, tmp, **kwargs)
+                return super(TempPath, cls).__new__(cls, p.generate(**kw), **kwargs)
             return super(TempPath, cls).__new__(cls, p, **kwargs)
         else:
-            sp = Path(*parts)
+            sp = p.joinpath(*parts)
             if sp.is_under(p):
-                return super(TempPath, cls).__new__(cls, sp if sp.is_dir() else sp.dirname, **kwargs)
+                return super(TempPath, cls).__new__(cls, sp, **kwargs)
             raise ValueError("The given path shall be under '{}'".format(p))
     
+    def tempdir(self, dirname=None, **kwargs):
+        """ Create and return a TempPath subdirectory instance. """
+        return TempPath(self.generate(**kwargs) if dirname is None else self.joinpath(dirname))
+    
     def tempfile(self, filename=None, **kwargs):
         """ Instantiate a NamedTemporaryFile or use an existing file in the TempPath and return it as a Path object. """
-        return Path(TempFile(dir=str(self), **kwargs).name) if filename is None else self.joinpath(filename)
+        return Path(TempFile(dir=str(self), **kwargs).name if filename is None else self.joinpath(filename))
 
