@@ -24,8 +24,9 @@ from .helpers.text import configure_docformat, gt
 
 __all__ = __features__ = ["parser", "initialize", "validate"]
 
-AT_EXIT_SET = False
-BANNER_FONT = None
+AT_EXIT_SET  = False
+BANNER_ARG   = None
+BANNER_FONT  = None
 BANNER_STYLE = {}
 
 parser_calls = []  # will be populated by calls to ProxyArgumentParser
@@ -64,24 +65,19 @@ def initialize(add_banner=False,
 
     :param add_banner:          add an ASCII banner when starting the tool
     :param add_config:          add an option to input an INI configuration file
-    :param add_demo:            add an option to re-run the process using a
-                                 random entry from the __examples__ (only works
-                                 if this variable is populated)
+    :param add_demo:            add an option to re-run the process using a random entry from the __examples__
+                                 (only works if this variable is populated)
     :param add_interact:        add an interaction option
     :param add_notify:          add a notification option
     :param add_progress:        add a progress management option
     :param add_step:            add an execution stepping option
     :param add_time:            add an execution timing option
     :param add_version:         add a version option
-    :param add_wizard:          add an option to run a wizard, asking for each
-                                 input argument
-    :param sudo:                if True, require sudo credentials and re-run
-                                 script with sudo
-    :param multi_level_debug:   allow to use -v, -vv, -vvv (adjust logging
-                                 level) instead of just -v (only debug on/off)
+    :param add_wizard:          add an option to run a wizard, asking for each input argument
+    :param sudo:                if True, require sudo credentials and re-run script with sudo
+    :param multi_level_debug:   allow to use -v, -vv, -vvv (adjust logging level) instead of just -v (only debug on/off)
     :param short_long_help:     enable/disable the separation of -h/--help
-    :param action_at_interrupt: perform an action at interrupt
-                                 (confirm|continue|exit)
+    :param action_at_interrupt: perform an action at interrupt (confirm|continue|exit)
     :param ext_logging:         extended logging options
     :param noargs_action:       action to be performed when no argument is input
     :param post_actions:        enable post-actions at interrupt
@@ -284,8 +280,11 @@ def initialize(add_banner=False,
         exe = ["runas", "/env", "/user:Administrator"] if WINDOWS else ["sudo", "-E"]
         os.execvp(["sudo", "runas"][WINDOWS], exe + [sys.executable] + sys.argv)
     # 4) configure logging and get the main logger
-    configure_logger(glob, multi_level_debug, glob['args']._collisions.get("relative"),
-                     glob['args']._collisions.get("logfile"), glob['args']._collisions.get("syslog"))
+    a = glob['args']
+    configure_logger(glob, multi_level_debug,
+                     getattr(a, a._collisions.get("relative", "relative"), False),
+                     getattr(a, a._collisions.get("logfile", "logfile"), None),
+                     getattr(a, a._collisions.get("syslog", "syslog"), None))
     # 5) append modes items
     set_interact_items(glob)
     set_hotkeys(glob)
@@ -297,9 +296,8 @@ def initialize(add_banner=False,
     bf = glob.get('BANNER_FONT', BANNER_FONT)
     if add_banner or isinstance(bf, string_types):
         f = AsciiFile()
-        bs = BANNER_STYLE
-        bs = bs if isinstance(bs, dict) else {}
-        f['title', bs] = Banner(p.scriptname, font=bf)
+        banner = getattr(glob['args'], glob.get('BANNER_ARG', BANNER_ARG) or "", None) or p.scriptname
+        f['title', glob.get('BANNER_STYLE', BANNER_STYLE)] = Banner(banner, font=bf)
         print(f)
     # 7) finally, bind the global exit handler
     _hooks.sigint_action = action_at_interrupt if exit_at_interrupt is None else ["continue", "exit"][exit_at_interrupt]
